package driver;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import indexing.BitSetDecorationFactory;
import indexing.GSTFactory;
import indexing.IndexStructureFactory;
import indexing.NodeDecorationFactory;

import input.GeneFamily;
import motifalgorithms.DeNovoExactDiscoveryAlgorithm;
import motifalgorithms.DevNullContainer;
import motifalgorithms.DiscoveryAlgorithm;
import motifalgorithms.MotifSearchSpace;
import motifmodels.IUPACFactory;
import phylogenetics.BLS;
import phylogenetics.BLSCalculator;
import alphabets.Alphabet;
import alphabets.IUPACAlphabet;
import alphabets.IUPACAlphabet.IUPACType;

public class RetrieveStatsForASingleGF {

	public static final String baseChars="ACGT";
	public static final boolean withReverseComplements = true;

	public static void main(String [] args) throws IOException {

		//meaningful bls cutoffs: 15,50,60,70,90
		int [] blsTresholds = {10,50,60,70,90};
		BLS.initializeBLSConstants(blsTresholds); 
		
		String filename = args[0];
		BufferedReader in=new BufferedReader(new FileReader(filename));
		
		GeneFamily gf = new GeneFamily(in);
		DiscoveryAlgorithm alg = new DeNovoExactDiscoveryAlgorithm(gf.getSequences());
			
		int kmax=12;
		
		//a)
		
		int maxDepth = kmax;
		boolean withReverseComplements=true;
		NodeDecorationFactory nodeDecoFac= new BitSetDecorationFactory();
		IndexStructureFactory iFac = new GSTFactory(maxDepth, withReverseComplements, nodeDecoFac);
		alg.setDataStructure(iFac);
		
		//b)
		
		int kmin=6;
		int maxDegPos=3;
		Alphabet alph = new IUPACAlphabet(IUPACType.TWOFOLDSANDN);
		MotifSearchSpace searchSpace = new MotifSearchSpace(kmin,kmax,maxDegPos,alph);
		
		alg.setSearchSpace(searchSpace);
		
		//c)
		
		BLSCalculator calculator=new BLSCalculator(gf);
		calculator.setCutoff(new BLS(BLS.MIN));
		alg.setConservationScoreCalculator(calculator);
		
		//d)
		
		MotifExtractor extractor = new DevNullContainer();
		alg.setMotifExtractor(extractor);
		
		long start=System.nanoTime();
		alg.runDiscovery(new IUPACFactory(IUPACType.TWOFOLDSANDN));
		long stop=System.nanoTime();
		
		
		double numMilliSecs=(stop-start)/1000000; 
		int numberOfMotifs = extractor.getNumberOfMotifsExtracted();
		
		//alg lopen: aantal motieven / tijd voor discovery
		
		System.out.println(filename.substring(0,filename.length()-4) + "\t" + numberOfMotifs + "\t" + numMilliSecs);
		
	}

}
